#include<stdlib.h>
#include<stdio.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>

#include "FuzzyInfEngine.h"
#include "FuzzyVariable.h"
#include "FuzzySet.h"
#include "gaussMF.h"
#include "sigmoidMF.h"
#include "trapMF.h"
#include "Centroid_Area.h"
#include "Bisector_Area.h"
#include "Small_Maximum.h"
#include "Large_Maximum.h"
#include "Mean_Maximum.h"
#include <math.h>


void write_func(Function* func, char *fileName)
{  ofstream outF1(fileName);
 
   for(float k=0.0;k<1.0;k+=0.01) 
        outF1 << k << " " << (*func).getValue(k) <<endl;
    
   outF1.close();

}

void write_func2(char *fileName,float cnr, float sp)
{  ofstream outF1(fileName);
 
   for(float k=0.0;k<1.0;k+=0.01) 
   {
      double val = 1.0/(1.0+exp(-1.0*sp*(k-cnr)));
        outF1 << k << " " << val <<endl;
   }
    
   outF1.close();

}

void main1()
{ gaussMF small(0.0,0.2);
  FuzzySet st(&small);
  FuzzyVariable X(st);

  write_func(&small,"small.X");

  gaussMF medium(0.5,0.4);
  st.setMFunction(&medium);
  X.addClass(st);

  write_func(&medium,"medium.X");

  gaussMF large(0.99,0.2);
  st.setMFunction(&large);
  X.addClass(st);

  write_func(&large,"large.X");

  gaussMF smallY(0.2,0.3);
  st.setMFunction(&smallY);
  FuzzyVariable Y(st);
  write_func(&smallY,"small.Y");


  gaussMF mediumY(0.6,0.3);
  st.setMFunction(&mediumY);
  Y.addClass(st);
  write_func(&mediumY,"medium.Y");

  gaussMF largeY(0.95,0.3);
  st.setMFunction(&largeY);
  Y.addClass(st);
  write_func(&largeY,"large.Y");

  cout << X << endl;
  cout << Y << endl;

  FuzzyInfEngine myEngine(1,1);

  float in[1];   

  in[0]=1;

  Vector inp1(in,1);
  Vector out1(in,1);
  myEngine.addRule(inp1,out1);
  //cout << "after first rule " << myEngine;

  in[0]=2;
  Vector inp2(in,1);
  Vector out2(in,1);
  cout << "after first rule " << myEngine;
  myEngine.addRule(inp2,out2);

  in[0]=3;
  Vector inp3(in,1);
  Vector out3(in,1);
  myEngine.addRule(inp3,out3);

  cout << myEngine;

  myEngine.addInput(&X);
  myEngine.addOutput(&Y);

  in[0]=0.3;

  Vector try1(in,1);
  //Centroid_Area def; 
  Bisector_Area def(100); 
  //Small_Maximum def(100); 
  //Large_Maximum def(100); 
  //Mean_Maximum def(100); 
  cout << " Input " << try1 << " output : " << myEngine.RunEngine(try1,&def) << endl;
}

main()
{  sigmoidMF none(0.05,-100.0);
   trapMF    small(0.05,0.08,0.09,0.13);
   trapMF    large(0.10,0.18,0.25,0.35);
   sigmoidMF VLarge(0.4,25.0);
   
   write_func(&none,"none.M");
   write_func(&small,"small.M");
   write_func(&large,"large.M");
   write_func(&VLarge,"vlarge.M");

   sigmoidMF noneS(0.1,-100.0);
   trapMF    smallS(0.09,0.18,0.23,0.29);
   trapMF    largeS(0.23,0.35,0.42,0.49);
   sigmoidMF VLargeS(0.5,25.0);

   write_func(&noneS,"noneS.M");
   write_func(&smallS,"smallS.M");
   write_func(&largeS,"largeS.M");
   write_func(&VLargeS,"vlargeS.M");

   gaussMF decrease(0.2,0.2);
   gaussMF stay(0.5,0.1);
   gaussMF increase(0.8,0.2);

   FuzzySet st(&none);
   FuzzyVariable NumExactMatches(st);
   st.setMFunction(&small);
   NumExactMatches.addClass(st);
   st.setMFunction(&large);
   NumExactMatches.addClass(st);
   st.setMFunction(&VLarge);
   NumExactMatches.addClass(st);

   st.setMFunction(&noneS);
   FuzzyVariable SofD(st);
   st.setMFunction(&smallS);
   SofD.addClass(st);
   st.setMFunction(&largeS);
   SofD.addClass(st);
   st.setMFunction(&VLargeS);
   SofD.addClass(st);

   st.setMFunction(&decrease);
   FuzzyVariable CrossOperation(st);
   st.setMFunction(&stay);
   CrossOperation.addClass(st);
   st.setMFunction(&increase);
   CrossOperation.addClass(st);

   st.setMFunction(&decrease);
   FuzzyVariable MutaOperation(st);
   st.setMFunction(&stay);
   MutaOperation.addClass(st);
   st.setMFunction(&increase);
   MutaOperation.addClass(st);

   FuzzyInfEngine myEngine(2,2);
   myEngine.addInput(&NumExactMatches);
   myEngine.addInput(&SofD);

   myEngine.addOutput(&CrossOperation);
   myEngine.addOutput(&MutaOperation);

   float inrule1[2] = { 1.0, 1.0};
   float inrule2[2] = { 1.0, 2.0};
   float inrule3[2] = { 1.0, 3.0};
   float inrule4[2] = { 1.0, 4.0};

   float outrule1[2] = { 1.0, 1.0};
   float outrule2[2] = { 1.0, 2.0};
   float outrule3[2] = { 1.0, 3.0};
   float outrule4[2] = { 2.0, 1.0};
   float outrule5[2] = { 2.0, 2.0};
   float outrule6[2] = { 2.0, 3.0};
   float outrule7[2] = { 3.0, 3.0};

   Vector inp1(inrule1,2);
   Vector out1(outrule4,2);
   myEngine.addRule(inp1,out1);

   Vector inp2(inrule2,2);
   Vector out2(outrule4,2);
   myEngine.addRule(inp2,out2);

   Vector inp3(inrule3,2);
   Vector out3(outrule1,2);
   myEngine.addRule(inp3,out3);

   Vector inp4(inrule4,2);
   Vector out4(outrule2,2);
   myEngine.addRule(inp4,out4);

   float inrule5[2] = { 2.0, 1.0};
   float inrule6[2] = { 2.0, 2.0};
   float inrule7[2] = { 2.0, 3.0};
   float inrule8[2] = { 2.0, 4.0};

   Vector inp5(inrule5,2);
   Vector out5(outrule3,2);
   myEngine.addRule(inp5,out5);

   Vector inp6(inrule6,2);
   Vector out6(outrule5,2);
   myEngine.addRule(inp6,out6);

   Vector inp7(inrule7,2);
   Vector out7(outrule5,2);
   myEngine.addRule(inp7,out7);

   Vector inp8(inrule8,2);
   Vector out8(outrule5,2);
   myEngine.addRule(inp8,out8);

   float inrule9[2] = { 3.0, 1.0};
   float inrule10[2] = { 3.0, 2.0};
   float inrule11[2] = { 3.0, 3.0};
   float inrule12[2] = { 3.0, 4.0};

   Vector inp9(inrule9,2);
   Vector out9(outrule7,2);
   myEngine.addRule(inp9,out9);

   Vector inp10(inrule10,2);
   Vector out10(outrule7,2);
   myEngine.addRule(inp10,out10);

   Vector inp11(inrule11,2);
   Vector out11(outrule6,2);
   myEngine.addRule(inp11,out11);

   Vector inp12(inrule12,2);
   Vector out12(outrule5,2);
   myEngine.addRule(inp12,out12);

   float inrule13[2] = { 4.0, 1.0};
   float inrule14[2] = { 4.0, 2.0};
   float inrule15[2] = { 4.0, 3.0};
   float inrule16[2] = { 4.0, 4.0};

   Vector inp13(inrule13,2);
   Vector out13(outrule7,2);
   myEngine.addRule(inp13,out13);

   Vector inp14(inrule14,2);
   Vector out14(outrule7,2);
   myEngine.addRule(inp14,out14);

   Vector inp15(inrule15,2);
   Vector out15(outrule7,2);
   myEngine.addRule(inp15,out15);

   Vector inp16(inrule16,2);
   Vector out16(outrule6,2);
   myEngine.addRule(inp16,out16);

   cout << " My engine has " << myEngine << endl;


   float test11[2] = { 0.01, 0.1};
   float test12[2] = { 0.01, 0.2};
   float test13[2] = { 0.01, 0.3};
   float test14[2] = { 0.01, 0.6};

   float test21[2] = { 0.1, 0.1};
   float test22[2] = { 0.1, 0.2};
   float test23[2] = { 0.1, 0.3};
   float test24[2] = { 0.1, 0.6};

   float test31[2] = { 0.22, 0.1};
   float test32[2] = { 0.22, 0.2};
   float test33[2] = { 0.22, 0.3};
   float test34[2] = { 0.22, 0.6};

   float test41[2] = { 0.6, 0.1};
   float test42[2] = { 0.6, 0.2};
   float test43[2] = { 0.6, 0.3};
   float test44[2] = { 0.6, 0.6};

   Vector **tests = new Vector*[16];

   tests[0] = new Vector(test11,2);
   tests[1] = new Vector(test12,2);
   tests[2] = new Vector(test13,2);
   tests[3] = new Vector(test14,2);

   tests[4] = new Vector(test21,2);
   tests[5] = new Vector(test22,2);
   tests[6] = new Vector(test23,2);
   tests[7] = new Vector(test24,2);

   tests[8] = new Vector(test31,2);
   tests[9] = new Vector(test32,2);
   tests[10] = new Vector(test33,2);
   tests[11] = new Vector(test34,2);

   tests[12] = new Vector(test41,2);
   tests[13] = new Vector(test42,2);
   tests[14] = new Vector(test43,2);
   tests[15] = new Vector(test44,2);

   Centroid_Area def; 
  //Bisector_Area def(100); 
  //Small_Maximum def(100); 
  //Large_Maximum def(100); 
  //Mean_Maximum def(100); 
   Vector res(test41,2);

   for(int i=0;i<16;i++)
   {  cout << " let's test " << *tests[i] << endl; 
      res = myEngine.RunEngine(*tests[i],&def);
      
      cout << i+1 << " test " << res << endl;

      cout << " decrease " << decrease.getValue(res.getElement(0)) << endl;
      cout << " keep " << stay.getValue(res.getElement(0)) << endl;
      cout << " increase " << increase.getValue(res.getElement(0)) << endl;

      cout <<  *tests[i] << " action CROSSOVER is " ;
      switch ( CrossOperation.getClassNumber(res.getElement(0)) )
      {  case 1 : cout << " DECREASE " << endl;
		  break;
         case 2 : cout << " KEEP " << endl;
		  break;
         case 3 : cout << " INCREASE " << endl;
		  break;
         default : cout << " default " << endl;
		  break;
      }// end of switch

      cout <<  *tests[i] << " action for MUTATION is " ;
      switch ( MutaOperation.getClassNumber(res.getElement(1)) )
      {  case 1 : cout << " DECREASE " << endl;
		  break;
         case 2 : cout << " KEEP " << endl;
		  break;
         case 3 : cout << " INCREASE " << endl;
		  break;
         default : cout << " default " << endl;
		  break;
      }// end of switch

   }// end of for

   cout << (*tests[0]).getElement(0) << " none : " << none.getValue((*tests[0]).getElement(0)) << endl;
   cout << " small : " << small.getValue((*tests[0]).getElement(0)) << endl;
   cout << " large : " << large.getValue((*tests[0]).getElement(0)) << endl;
   cout << " Vlarge : " << VLarge.getValue((*tests[0]).getElement(0)) << endl;

   cout << (*tests[0]).getElement(1) << " noneS : " << noneS.getValue((*tests[0]).getElement(1)) << endl;
   cout << " smallS : " << smallS.getValue((*tests[0]).getElement(1)) << endl;
   cout << " largeS : " << largeS.getValue((*tests[0]).getElement(1)) << endl;
   cout << " VlargeS : " << VLargeS.getValue((*tests[0]).getElement(1)) << endl;

}// end of main
